"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tsserverlibrary_1 = require("typescript/lib/tsserverlibrary");
function create(config) {
    return new ToolImpl(config);
}
exports.create = create;
class ToolImpl {
    constructor(config) {
        this.config = config;
        this.config.prefix = this.config.prefix || '&%&%';
        this.buildEvalExpr();
    }
    buildEvalExpr() {
        this.allActionsEvalPrefix = '';
        // want to get to an string like this: function moveThisFileTo(path){return {name: 'moveThisFileTo', args: {dest: path} }};
        this.config.actions.forEach(actionConfig => {
            this.allActionsEvalPrefix += `function ${actionConfig.name}(${actionConfig.args.join(', ')}) {return {name: '${actionConfig.name}', args: {${actionConfig.args.join(', ')}}}};\n`;
        });
    }
    /**
     * find all declared actions in given file string
     * @param fileStr content of file where to find action declarations - in general obtained with `sourceFile.getText()`
     */
    findActions(fileStr) {
        const found = [];
        const lines = fileStr.split('\n'); //TODO: use new line format in tsconfig
        lines.forEach(line => {
            const i = line.indexOf(this.config.prefix);
            if (i === -1) {
                return;
            }
            const userCall = line.substr(i + this.config.prefix.length, line.length);
            try {
                const result = eval(this.allActionsEvalPrefix + ';\n' + userCall);
                if (result && typeof result === 'object') {
                    const actionConfig = this.config.actions.find(a => userCall.trim().startsWith(a.name)); // TODO: issue if two start with the same prefix 
                    result.print = actionConfig.print.bind(result);
                    found.push(result);
                }
            }
            catch (ex) {
                // console.log(ex, ex.stack)
            }
        });
        return found;
    }
    getCompletionsAtPosition(fileName, position, options) {
        return this.config.actions.map(action => ({
            name: `refactor.${action.name}`,
            kind: tsserverlibrary_1.ScriptElementKind.unknown,
            kindModifiers: tsserverlibrary_1.ScriptElementKindModifier.none,
            sortText: `refactor.${action.name}`,
            insertText: `// ${this.config.prefix} ${action.snippet}`
        }));
    }
}
//# sourceMappingURL=index.js.map